% simuate ShiftCP data
clear all;
Fs=512;  % Sample rate
time_points=linspace(0,1,Fs)';
%% Generate non-negative trial strenghts
FACT{1}=rand(10,4);


%% Generate Drift component
FACT{2}(:,1)=time_points-mean(time_points);
% Generate 24 Hz component
FACT{2}(:,2)=cos(2*pi*24*time_points*Fs);
% Generate 12 Hz component
FACT{2}(:,3)=cos(2*pi*12*time_points*Fs);
% Generate 150 Hz component
FACT{2}(:,4)=cos(2*pi*50*time_points*Fs);
% Normalize generated time components
FACT{2}=FACT{2}./(ones(size(FACT{2},1),1)*sqrt(sum(FACT{2}.^2)));

%% Generate random topography
FACT{3}=randn(10,4);

%% Generate delays between -10 and 10 samples
T=ceil(20*rand(10,4)-10);

%% Reconstruct the data and add noise
Rec = reconstructShiftCP(FACT,T); % Generate Data
X=Rec+0.01*randn(size(Rec));   % Add noise

% Set algorithm parameters
opts.maxiter=250;
opts.constr=[1 0 0];
opts.SNR=0;
opts.Sample_rate=Fs;
opts.ARD=true;      
opts.InitRun=3;

figure;
noc=10;
for k=1:3
    if k==2 
        % Remove frequencies around 12 Hz alpha oscillations and 50 Hz line
        % noise from all components
        for rep=1:noc
            opts.RemoveFrequencies{rep}=[49.5 50.5]; 
        end
    elseif k==3
         % Remove frequencies around 12 Hz alpha oscillations and 50 Hz
         % line noise from all components
        for rep=1:noc
            opts.RemoveFrequencies{rep}=[11.5 12.5; 49.5 50.5];
        end
    end
    
    % Run the algorithm
    [FACT_est{k},T_est{k},cost,varexpl,Lambda] = ShiftCP(X,10,opts);
    
    if k==1
        subplot(4,1,1);
        plot(FACT{2},'linewidth',2);
        title('True factors','FontWeight','bold');
        axis tight;
    end
    subplot(4,1,k+1);    
    plot(FACT_est{k}{2},'linewidth',2);
    
    axis tight;
    if k==1
        title('Estimated factors no constraints','FontWeight','bold');
    elseif k==2
        title('Estimated factors constrained for 50 Hz component','FontWeight','bold');
    elseif k==3
        title('Estimated factors constrained for 12 and 50 Hz components','FontWeight','bold');
    end
end
