%----------------------------------------------------------
function [A,mu,cost]=nnpg(XSt,SSt,Aold,mu,maxiter)
%   Gradient Based Sparse Coding
%       solves
%           argmin_A 0.5||X-AS||_F^2
%
%   Usage:

%
%   Input
%           XSt             X*S'
%           SSt             S*S'
%           Aold            Initial value of A
%           mu              initial step size
%           maxiter            maximal number of iterations
%
%   Output
%           A               solution matrix
%           mu              step size
%           cost         0.5*(||X-AS||_F^2-||X||_F^2)


tol=1e-9;
dcost=inf;
cost=-sum(sum(XSt.*Aold))+0.5*sum(sum((Aold'*Aold).*SSt));
N=size(Aold,1);
k=1;
while k<maxiter && dcost>tol
    cost_old=cost;
    grad=(Aold*SSt-XSt)/(trace(SSt)/size(SSt,1));     
    stop=0;
    while ~stop 
      A=Aold-mu*grad;
      A(A<0)=0;      
      cost=-sum(sum(XSt.*A))+0.5*sum(sum((A'*A).*SSt));        
      dcost=cost_old-cost;
      if dcost>=0
           mu=1.2*mu;
           stop=1;           
           Aold=A;
      else
           mu=mu/2;
      end
    end
    k=k+1;
end
